﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;
using System.Drawing.Design;
using System.Net;
using System.IO;

/// <summary>
/// This project is developed by Andrey A. Fedorov from BizKit.ru.
/// E-mail me at fedorov@bizkit.ru for any support on this code or even support on BigBlueButton.
/// </summary>
namespace BigBlueButtonSDK
{
    public class CreateMeetingRequest : BaseRequest
    {
        public CreateMeetingRequest()
            : base()
        {
            this._methodName = "create";
            this._response = new CreateMeetingResponse(); 
        }

        public new CreateMeetingResponse Execute()
        {
            return (CreateMeetingResponse)base.Execute();
        }

        /// <summary>
        /// Optional    A name for the meeting.
        /// </summary>
        [DisplayName("Meeting name (optional)")]
        [Description("A name for the meeting.")]
        public string Name
        {
            get { return GetParamByName("name"); }
            set { AddParameters("name", value); }
        }

        /// <summary>
        /// Required	A meeting ID that can be used to identify this meeting by the 3rd-party application.
        /// </summary>
        [DisplayName("Meeting ID (required)")]
        [Description("A meeting ID that can be used to identify this meeting by the 3rd-party application.")]
        public string MeetingID
        {
            get 
            { 
                string meetingID = GetParamByName("meetingID"); 
                if (String.IsNullOrEmpty(meetingID))
                {
                    meetingID = Guid.NewGuid().ToString();
                    this.MeetingID = meetingID;
                }
                return meetingID;
            }
            set { AddParameters("meetingID", value); }
        }

        /// <summary>
        /// Optional	The password that will be required for attendees to join the meeting. 
        /// This is optional, and if not supplied, BBB will assign a random password.
        /// </summary>
        [DisplayName("Attendee Password (optional)")]
        [Description("The password that will be required for attendees to join the meeting. This is optional, and if not supplied, BBB will assign a random password.")]
        public string AttendeePassword
        {
            get { return GetParamByName("attendeePW"); }
            set { AddParameters("attendeePW", value); }
        }

        /// <summary>
        /// Optional	The password that will be required for moderators to join the meeting or for certain administrative actions (i.e. ending a meeting). 
        /// This is optional, and if not supplied, BBB will assign a random password.
        /// </summary>
        [DisplayName("Moderator Password (optional)")]
        [Description("The password that will be required for moderators to join the meeting or for certain administrative actions (i.e. ending a meeting). This is optional, and if not supplied, BBB will assign a random password.")]
        public string ModeratorPassword
        {
            get { return GetParamByName("moderatorPW"); }
            set { AddParameters("moderatorPW", value); }
        }

        /// <summary>
        /// Optional	A welcome message that gets displayed on the chat window when the participant joins. 
        /// You can include keywords (%%CONFNAME%%, %%DIALNUM%%, %%CONFNUM%%) which will be substituted automatically. 
        /// You can set a default welcome message on bigbluebutton.properties
        /// </summary>
        [DisplayName("Welcome (optional)")]
        [Description("A welcome message that gets displayed on the chat window when the participant joins. You can include keywords (%%CONFNAME%%, %%DIALNUM%%, %%CONFNUM%%) which will be substituted automatically. You can set a default welcome message on bigbluebutton.properties")]
        public string Welcome
        {
            get { return GetParamByName("welcome"); }
            set { AddParameters("welcome", value); }
        }

        /// <summary>
        /// Optional	The dial access number that participants can call in using regular phone. 
        /// You can set a default dial number on bigbluebutton.properties
        /// </summary>
        [DisplayName("Dial Number (optional)")]
        [Description("The dial access number that participants can call in using regular phone. You can set a default dial number on bigbluebutton.properties")]
        public string DialNumber
        {
            get { return GetParamByName("dialNumber"); }
            set { AddParameters("dialNumber", value); }
        }

        /// <summary>
        /// Optional	Voice conference number that participants enter to join the voice conference. 
        /// The default pattern for this is a 5-digit number. This is the PIN that a dial-in user must enter to join the conference. 
        /// If you want to change this pattern, you have to edit FreeSWITCH dialplan and defaultNumDigitsForTelVoice of bigbluebutton.properties. 
        /// When using the default setup, we recommend you always pass a 5-digit voiceBridge parameter. 
        /// Finally, if you don't pass a value for voiceBridge, then users will not be able to join a voice conference for the session.
        /// </summary>
        [DisplayName("Voice Bridge (optional)")]
        [Description("Voice conference number that participants enter to join the voice conference. The default pattern for this is a 5-digit number. This is the PIN that a dial-in user must enter to join the conference. If you want to change this pattern, you have to edit FreeSWITCH dialplan and defaultNumDigitsForTelVoice of bigbluebutton.properties. When using the default setup, we recommend you always pass a 5-digit voiceBridge parameter. Finally, if you don't pass a value for voiceBridge, then users will not be able to join a voice conference for the session.")]
        public string VoiceBridge
        {
            get { return GetParamByName("voiceBridge"); }
            set { AddParameters("voiceBridge", value); }
        }

        /// <summary>
        /// Optional	Voice conference alphanumberic that participants enter to join the voice conference.
        /// </summary>
        [DisplayName("Web voice (optional)")]
        [Description("Voice conference alphanumberic that participants enter to join the voice conference.")]
        public string WebVoice
        {
            get { return GetParamByName("webVoice"); }
            set { AddParameters("webVoice", value); }
        }

        /// <summary>
        /// Optional	The URL that the BigBlueButton client will go to after users click the OK button on the 'You have been logged out message'. 
        /// This overrides, the value for bigbluebutton.web.loggedOutURL if defined in bigbluebutton.properties
        /// </summary>
        [DisplayName("Logout URL (optional)")]
        [Description("The URL that the BigBlueButton client will go to after users click the OK button on the 'You have been logged out message'. This overrides, the value for bigbluebutton.web.loggedOutURL if defined in bigbluebutton.properties")]
        public string LogoutURL
        {
            get { return GetParamByName("logoutURL"); }
            set { AddParameters("logoutURL", value); }
        }

        /// <summary>
        /// Optional	Setting ‘record=true’ instructs the BigBlueButton server to record the media and events in the session for later playback. 
        /// Available values are true or false. Default value is false.
        /// </summary>
        [DisplayName("Record (optional)")]
        [Description("Setting ‘record=true’ instructs the BigBlueButton server to record the media and events in the session for later playback. Available values are true or false. Default value is false.")]
        public bool Record
        {
            get 
            { return StrToBool(GetParamByName("record")); }
            set { AddParameters("record", value.ToString()); }
        }

        /// <summary>
        /// Optional	The maximum length (in minutes) for the meeting. Normally, the BigBlueButton server will end the meeting when either 
        /// (a) the last person leaves (it takes a minute or two for the server to clear the meeting from memory) or 
        /// when the server receives an end API request with the associated meetingID (everyone is kicked and the meeting is immediately cleared from memory). 
        /// BigBlueButton begins tracking the length of a meeting when the first person joins. 
        /// If duration contains a non-zero value, then when the length of the meeting exceeds the duration value the server will immediately end the meeting (same as receiving an end API request).
        /// </summary>
        [DisplayName("Duration (optional)")]
        [Description("The maximum length (in minutes) for the meeting. Normally, the BigBlueButton server will end the meeting when either (a) the last person leaves (it takes a minute or two for the server to clear the meeting from memory) or when the server receives an end API request with the associated meetingID (everyone is kicked and the meeting is immediately cleared from memory). BigBlueButton begins tracking the length of a meeting when the first person joins. If duration contains a non-zero value, then when the length of the meeting exceeds the duration value the server will immediately end the meeting (same as receiving an end API request).")]
        public string Duration
        {
            get { return GetParamByName("duration"); }
            set { AddParameters("duration", value); }
        }

        /// <summary>
        /// Optional	You can pass one or more metadata values for create a meeting. 
        /// These will be stored by BigBlueButton and later retrievable via the getMeetingInfo call and getRecordings. 
        /// Examples of meta parameters are meta_Presenter, meta_category, meta_LABEL, etc. 
        /// All parameters are converted to lower case, so meta_Presenter would be the same as meta_PRESENTER.
        /// </summary>
        [DisplayName("Meta (optional)")]
        [Description("You can pass one or more metadata values for create a meeting. These will be stored by BigBlueButton and later retrievable via the getMeetingInfo call and getRecordings. Examples of meta parameters are meta_Presenter, meta_category, meta_LABEL, etc. All parameters are converted to lower case, so meta_Presenter would be the same as meta_PRESENTER.")]
        public string Meta
        {
            get { return GetParamByName("meta"); }
            set { AddParameters("meta", value); }
        }

        /// <summary>
        /// Optional	Display a message to all moderators in the public chat.
        /// </summary>
        [DisplayName("Moderator only message (optional)")]
        [Description("Display a message to all moderators in the public chat.")]
        public string ModeratorOnlyMessage
        {
            get { return GetParamByName("moderatorOnlyMessage"); }
            set { AddParameters("moderatorOnlyMessage", value); }
        }

        /// <summary>
        /// Optional	Automatically starts recording when first user joins. 
        /// NOTE: Don't set to autoStartRecording =false and allowStartStopRecording=false as the user won't be able to record.
        /// </summary>
        [DisplayName("Autostart recording (optional)")]
        [Description("Automatically starts recording when first user joins. NOTE: Don't set to autoStartRecording =false and allowStartStopRecording=false as the user won't be able to record.")]
        public bool AutoStartRecording
        {
            get { return StrToBool(GetParamByName("autoStartRecording")); }
            set { AddParameters("autoStartRecording", value.ToString()); }
        }

        /// <summary>
        /// Optional	Allow the user to start/stop recording. 
        /// This means the meeting can start recording automatically (autoStartRecording=true) with the user able to stop/start recording from the client.
        /// </summary>
        [DisplayName("Allow start/stop recording (optional)")]
        [Description("Allow the user to start/stop recording. This means the meeting can start recording automatically (autoStartRecording=true) with the user able to stop/start recording from the client.")]
        public bool AllowStartStopRecording
        {
            get { return StrToBool(GetParamByName("allowStartStopRecording")); }
            set { AddParameters("allowStartStopRecording", value.ToString()); }
        } 
    }

    public class CreateMeetingResponse : BaseResponse
    {
        public string MeetingToken
        {
            get { return GetValueByName("meetingToken"); }
        }

        public string MeetingID
        {
            get { return GetValueByName("meetingID"); }
        }

        public string AttendeePassword
        {
            get { return GetValueByName("attendeePW"); }
        }

        public string ModeratorPassword
        {
            get { return GetValueByName("moderatorPW"); }
        }

        public DateTime CreateTime
        {
            get
            {
                string createTime = GetValueByName("createTime");
                return General.MillisecondsToDateTime(createTime);

            }
        }

        public string CreateDate
        {
            get { return GetValueByName("createDate"); }
        }

        public bool HasUserJoined
        {
            get 
            {
                string hasUserJoined = GetValueByName("hasUserJoined").ToLower();
                return hasUserJoined == "true";
            }
        }

        public string Duration
        {
            get { return GetValueByName("duration"); }
        }

        public string HasBeenForciblyEnded
        {
            get { return GetValueByName("hasBeenForciblyEnded"); }
        }
    }

    public class CreateMeeting
    {
        private CreateMeetingRequest _request;
        [CategoryAttribute("Create Meeting Request")]
        [TypeConverter(typeof(ExpandableObjectConverter))]
        [DisplayName("Create meeting request")]
        public CreateMeetingRequest Request
        {
            get { return _request; }
            set { _request = value; }
        }

        private CreateMeetingResponse _response;
        [CategoryAttribute("Create Meeting Response")]
        [DisplayName("Create meeting response")]
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public CreateMeetingResponse Response
        {
            get { return _response; }
            set { _response = value; }
        }
    }
}
